/*
 * calccore/ccif.cc
 * 
 * Calc core interface, wrappers and other stuff. 
 * This is part of the AniVision project. 
 * 
 * Copyright (c) 2003 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "ccif.h"
#include <objdesc/omatrix.h>

#include <assert.h>


namespace CC
{

int CCIF_Setting::SetTimeRange(const Range & /*tr*/)
{
	assert(0);
	return(0);
}


int CCIF_Setting::ComputeAll()
{
	assert(0);
	return(0);
}


int CCIF_Setting::TimeStep()
{
	assert(0);
	return(1);  // 1 -> animation end
}


CCIF_Setting::CCIF_Setting()
{
}

CCIF_Setting::~CCIF_Setting()
{
}

//------------------------------------------------------------------------------

int CCIF_Object::ProvidesVal(OVal ov) const
{
	switch(ov)
	{
		case ValPos:    // fall
		case ValFront:  //   trough
		case ValUp:     //     to 
		case ValSpeed:  //       here
		case ValAccel:  //          ...
		case ValMatrix: return((vflags & VF_Location) ? 1 : 0);
		case ValTime:   return((vflags & VF_Time) ? 1 : 0);
		case ValActive: return((vflags & VF_Active) ? 1 : 0);
	}
	return(0);
}


int CCIF_Object::GetVal(OVal ov,int *store_here)
{
	// Just the default implementation. 
	if(!ProvidesVal(ov)) return(1);
	switch(ov)  // Fall through switch. 
	{
		case ValPos:
		case ValFront:
		case ValUp:
		case ValSpeed:
		case ValAccel:
		case ValMatrix:
		case ValTime:
			return(2);  // incompatible type
		case ValActive:
			*store_here=1;   // default: active
			return(0);
	}
	assert(0);
}


int CCIF_Object::GetVal(OVal ov,double *store_here)
{
	// Just the default implementation. 
	if(!ProvidesVal(ov)) return(1);
	switch(ov)  // Fall through switch. 
	{
		case ValPos:
		case ValFront:
		case ValUp:
		case ValSpeed:
		case ValAccel:
			for(int i=0; i<3; i++)
			{  store_here[i]=0.0;  }
			return(0);
		case ValTime:
			*store_here=0.0;
			return(0);
		case ValMatrix:
		case ValActive:
			return(2);  // incompatible type
	}
	assert(0);
}


int CCIF_Object::GetVal(OVal ov,NUM::ObjectMatrix *store_here)
{
	// Just the default implementation. 
	if(!ProvidesVal(ov)) return(1);
	switch(ov)  // Fall through switch. 
	{
		case ValPos:
		case ValFront:
		case ValUp:
		case ValSpeed:
		case ValAccel:
		case ValTime:
		case ValActive:
			return(2);  // incompatible type
		case ValMatrix:
			*store_here=NUM::ObjectMatrix::IdentMat;
			return(0);
	}
	assert(0);
}


int CCIF_Object::EvalAniDescBlock(ADTNAniDescBlock * /*adb*/,ExecThread *et)
{
	// This function must be overruidden. 
	assert(et);
	assert(0);
	return(0);
}


CCIF_Object::CCIF_Object(CCIF_Setting *_setting,int _vflags)
{
	setting=_setting;
	vflags=_vflags;
}

CCIF_Object::~CCIF_Object()
{
}

//------------------------------------------------------------------------------

CCIF_Value::CCIF_Value(CCIF_Setting *_setting,int _dimension)
{
	setting=_setting;
	dimension=_dimension;
}

CCIF_Value::~CCIF_Value()
{
}

//------------------------------------------------------------------------------

const char *_onf_str="OOPS: Called %s for >NULL< factory.\n";

CCIF_Setting *CCIF_Factory::CreateCCIF_Setting()
{
	fprintf(stderr,_onf_str,"CreateCCIF_Object()");
	return(NULL);
}

CCIF_Object *CCIF_Factory::CreateCCIF_Object(CCIF_Setting *setting,
	int vflags)
{
	fprintf(stderr,_onf_str,"CreateCCIF_Object()");
	return(NULL);
}

CCIF_Value *CCIF_Factory::CreateCCIF_Value(CCIF_Setting *setting,
	int dimension)
{
	fprintf(stderr,_onf_str,"CreateCCIF_Value()");
	return(NULL);
}


CCIF_Factory::CCIF_Factory(const char *_name)
{
	name=_name;
}

CCIF_Factory::~CCIF_Factory()
{
	/* empty */
}

//------------------------------------------------------------------------------

void _ccif_failassert(int line)
{
	fprintf(stderr,"_ccif_failassert(%d)\n",line);
	abort();
}

}  // end of namespace CC
