/*
 * core/test.cc
 * 
 * Experimental main program for testing purposes. 
 * This is part of the AniVision project. 
 * 
 * Copyright (c) 2003--2004 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include <ani-parser/ani_parser.h>
#include <ani-parser/treereg.h>
#include <core/animation.h>

#include <stdio.h>
#include <stdlib.h>

//#include <ani-parser/exprtype.h>

#include <hlib/htime.h>
#include <../config.h>


char *prg_name=PACKAGE;

extern int hlib_allocdebug_trace;
extern size_t hlib_allocdebug_abort_seq;
extern void (*hlib_allocdebug_checkhdl)(void);

namespace ANI { extern int _test(); }
int main(int argc,char **arg)
{
	fprintf(stderr,"** This is AniVision-%s **\n",VERSION);
	
	// This only exists if allocation debugging is enabled in HLib: 
	// (Linker error otherwise.) 
	//hlib_allocdebug_trace=1;
	// Set this to find desired allocation call with debugger. 
	//hlib_allocdebug_abort_seq=1000;
	//hlib_allocdebug_checkhdl=&ANI::SomeFunction;
	
	fprintf(stderr,"sizeof(EV)=%d; sizeof(IEV)=%d\n",
		sizeof(ANI::ExprValue),sizeof(ANI::InternalExprValue));
	
	ANI::Ani_Parser *a_parser;
	AniAnimation *ani=NULL;
	do
	{
		fprintf(stderr,"NOTE: You are running a preliminary test parser."
			" (test.cc)\n");
		a_parser=new ANI::Ani_Parser();
		
		// Static initialisation. IMPORTANT: 
		ANI::ExprValueType::_InitStaticVals();
		ANI::OpFunc_InitStaticVals();
		
		HTime start(HTime::Curr);
		RefString str_or_file;
		str_or_file.set((argc<2) ? "grammar-test.txt" : arg[1]);
		HTime end(HTime::Curr);
		
ANI::_test();
		
		int rv=a_parser->ParseAnimation(str_or_file,/*is_file=*//*(argc<2) ?*/ 1 /*: 0*/);
		fprintf(stderr,"AMI: Parse=%d   (elapsed: %s)\n",rv,
			(end-start).PrintElapsed(1));
		
		/*fprintf(stdout,",----------------------------<complete source dump>----------------------------.\n");
		a_parser->DumpTree(stdout);
		fprintf(stdout,"`------------------------------------------------------------------------------'\n");*/
		
		// If there were no errors, perform registration step: 
		if(rv!=0) break;
		
		start=HTime::Curr;
		RefString ani_name;
		if(ani_name.set("[animation]"))
		{  CheckMalloc(NULL);  }
		ani=new AniAnimation(ani_name,a_parser->GetRootNode());
		
		rv=a_parser->DoRegistration(ani);
		end=HTime::Curr;
		
		fprintf(stderr,"ANI: DoRegistration=%d   (elapsed: %s)\n",rv,
			(end-start).PrintElapsed(1));
		int n_incomplete=0,n_scopes=ani->CountScopes(n_incomplete);
		fprintf(stderr,"ANI:   Number of ani scopes: %d  "
			"(and %d incomplete place holders)\n",n_scopes,n_incomplete);
		
		// Dump ani object tree: (before type fixup)
		/*{
			StringTreeDump strtd;
			ani->AniTreeDump(&strtd);
			fprintf(stdout,",---------------------------<complete ani scope dump>--------------------------.\n");
			strtd.Write(stdout);
			fprintf(stdout,"`------------------------------------------------------------------------------'\n");
		}*/
		
		// Dump after optimization: 
		//fprintf(stdout,",----------------------------<complete source dump>----------------------------.\n");
		//a_parser->DumpTree(stdout);
		//fprintf(stdout,"`------------------------------------------------------------------------------'\n");
		
		if(rv)  break;
		
		// We'd now need some constant folding code to solve all type 
		// names and array sizes of member vars. 
		// This actually still needs to be implemented. Currently, only 
		// REAL constants are accepted. 
		start=HTime::Curr;
		rv=ani->FixIncompleteTypes();
		end=HTime::Curr;
		fprintf(stderr,"AMI: FixIncompleteTypes=%d   (elapsed: %s)\n",rv,
			(end-start).PrintElapsed(1));
		
		// Dump ani object tree: (after incomplete type fixup)
		/*{
			StringTreeDump strtd;
			ani->AniTreeDump(&strtd);
			fprintf(stdout,",---------------------------<complete ani scope dump>--------------------------.\n");
			strtd.Write(stdout);
			fprintf(stdout,"`------------------------------------------------------------------------------'\n");
		}*/
		
		if(rv)  break;
		
		// Perform instance info creation step: 
		fprintf(stderr,"ANI: Creating instance info entries...\n");
		rv=ani->PerformCleanup(/*cstep=*/2);  // CSTEP 2
		fprintf(stderr,"ANI:   Instance info entries: %d errors\n",rv);
		
		if(rv)  break;
		
		// We now perform the (complete) type fixup step. 
		start=HTime::Curr;
		rv=a_parser->PerformExprTF();
		end=HTime::Curr;
		fprintf(stderr,"ANI: PerformExprTF=%d   (elapsed: %s)\n",rv,
			(end-start).PrintElapsed(1));
		int n_incomplete_before=0;
		n_scopes=ani->CountScopes(n_incomplete_before);
		ani->PerformCleanup(/*cstep=*/1);  // CLEANUP STEP 1
		n_incomplete=0;
		n_scopes=ani->CountScopes(n_incomplete);
		fprintf(stderr,"AMI:   Ani scopes: %d  "
			"(+%d incomplete; %d before cleanup)\n",
			n_scopes,n_incomplete,n_incomplete_before);
		if(rv)  break;
		
		fprintf(stdout,",----------------------------<complete source dump>----------------------------.\n");
		a_parser->DumpTree(stdout);
		fprintf(stdout,"`------------------------------------------------------------------------------'\n");
		
		// Dump ani object tree: (after complete type fixup)
		{
			StringTreeDump strtd;
			ani->AniTreeDump(&strtd);
			fprintf(stdout,",---------------------------<complete ani scope dump>--------------------------.\n");
			strtd.Write(stdout);
			fprintf(stdout,"`------------------------------------------------------------------------------'\n");
		}
		
		// For now (testing purposes), execute the "main()" routine. 
		ani->ExecMain();
	} while(0);
	
	// Here... to make alloc debugging happy. 
	
	// This will delete the main TreeNode tree: 
	// NOTE: Main tree gets deleted first and issues CG_DeleteNotify() 
	//       for attached ANI nodes...
	// Reason: One Ani scope can be attached to more than one TreeNode, 
	// hence the ani scope cannot do correct cleanup regarding the 
	// TreeNode pointers. 
	if(a_parser) delete a_parser;
	// This will delete the ANI tree: 
	if(ani) delete ani;
	// CLEANUP static data: 
	ANI::OpFunc_CleanupStaticVals();
	ANI::ExprValueType::_CleanupStaticVals();
	
	//fprintf(stderr,"ANI: %u bytes for %d tree nodes allocated.\n",
	//	ANI::TreeNode::tree_nodes_tot_size,ANI::TreeNode::n_alloc_tree_nodes);
	
	//fprintf(stderr,"sizeof(TreeNode)=%u\n",sizeof(ANI::TreeNode));
	
	LMallocUsage lmu;
	LMallocGetUsage(&lmu);
	fprintf(stderr,"%sLMalloc: used=%u/%d (max=%u) [%u,%u,%u]%s\n",
		lmu.curr_used ? "**** " : "",
		lmu.curr_used,lmu.used_chunks,
		lmu.max_used,
		lmu.malloc_calls,lmu.realloc_calls,lmu.free_calls,
		lmu.curr_used ? "**** " : "");
	
	return(0);
}
