/*
 * objdesc/curve_fixed.cc
 * 
 * Fixed "curve" class. 
 * This is part of the AniVision project. 
 * 
 * Copyright (c) 2004 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "curve_fixed.h"


namespace NUM
{

int CurveFunction_Fixed::Eval(double /*t*/,double *result) const
{
	for(int i=0; i<3; i++)
		result[i]=pos[i];
	return(0);
}

int CurveFunction_Fixed::EvalD(double /*t*/,double *result) const
{
	for(int i=0; i<3; i++)
		result[i]=0.0;
	return(0);
}

int CurveFunction_Fixed::EvalDD(double /*t*/,double *result) const
{
	for(int i=0; i<3; i++)
		result[i]=0.0;
	return(0);
}


int CurveFunction_Fixed::SetPos(double *_pos)
{
	if(!isnan(pos[0]))  return(1);
	for(int i=0; i<3; i++)
		pos[i]=_pos[i];
	return(0);
}


int CurveFunction_Fixed::Create()
{
	if(isnan(pos[0]))
	{
		// FIXME: If no pos is specified, should use continuacy. 
		pos[0]=pos[1]=pos[2]=0.0;
	}
	
	return(0);
}


CurveFunction_Fixed::CurveFunction_Fixed() : 
	CurveFunctionBase()
{
	for(int i=0; i<3; i++)
		pos[i]=NAN;
}

CurveFunction_Fixed::~CurveFunction_Fixed()
{
}

}  // end of namespace NUM
